package de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.instmodels;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.eclipse.emf.common.ui.dialogs.ResourceDialog;
import org.eclipse.emf.common.util.URI;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import de.mdelab.sdm.interpreter.code.debug.ui.EMFResourceSelectionDialog;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.ITableViewerListener;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.TableData;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.TableViewerWrapper;

/**
 * Table to edit story diagram instance models for Eclipse launch configuration
 */
public class InstanceModelTableViewer extends TableViewerWrapper<Object, InstanceModelRow, InstanceModelsColumnType> {
	
	private List<InstanceModelEditingSupport> editingSupports;

	public InstanceModelTableViewer( final Composite p_parent ) {
		super( p_parent, true );

		addViewerListener( new IInputModelsTableViewerListener() {
			
			@Override
			public void rowObjectUpdated( final InstanceModelRow p_modelRow ) {
				
				// Handle the case where the user sets the input model URI to null. Then the row must be deleted
				if ( p_modelRow.getInputInstanceModel() == null ) {
					removeRows( Collections.singletonList( p_modelRow ) );
				}
			}
			
			@Override
			public void rowObjectRemoved( final InstanceModelRow p_modelRow ) {
			}
			
			@Override
			public void rowObjectAdded( final InstanceModelRow p_modelRow ) {
			}

			@Override
			public void inputModelUpdated(InstanceModelRow p_modelRow) {
			}
		} );
	}
	
	@Override
	public void addViewerListener( final ITableViewerListener<InstanceModelRow> p_listener ) {
		for ( final InstanceModelEditingSupport support : editingSupports ) {
			support.addEditorListener( p_listener );
		}
		
		super.addViewerListener( p_listener );
	}
	
	@Override
	public boolean removeViewerListener( final ITableViewerListener<InstanceModelRow> p_listener ) {
		boolean removed = false;
		
		for ( final InstanceModelEditingSupport support : editingSupports ) {
			if ( support.removeEditorListener( p_listener ) ) {
				removed = true;
			}
		}

		return removed || super.removeViewerListener( p_listener );
	}
	
	protected void createColumns() {
		editingSupports = new ArrayList<InstanceModelEditingSupport>();
		
		InstanceModelEditingSupport instanceEditSupport = new InputInstanceModelEditingSupport( tableViewer );
		editingSupports.add( instanceEditSupport );
		createViewerColumn( 300, 
							"Input Model",
							new ColumnLabelProvider() {
								
								@Override
								public String getText( final Object p_element ) {
								    final InstanceModelRow row = (InstanceModelRow) p_element;
								    
								    return String.valueOf( row.getInputInstanceModel() );
								}
							},
							instanceEditSupport,
							InstanceModelsColumnType.INPUT );

		instanceEditSupport = new OutputInstanceModelEditingSupport( tableViewer );
		editingSupports.add( instanceEditSupport );
		createViewerColumn( 300, 
							"Output Model",
							new ColumnLabelProvider() {
								
								@Override
								public String getText( final Object p_element ) {
								    final InstanceModelRow row = (InstanceModelRow) p_element;
								    
								    return String.valueOf( row.getOutputInstanceModel() );
								}
							},
							instanceEditSupport,
							InstanceModelsColumnType.OUTPUT );
	}
	
	@Override
	protected InstanceModelComparator createTableViewerComparator( final ILabelProvider p_labelProvider ) {
		return new InstanceModelComparator( p_labelProvider );
	}
	
	public void initData( final List<InstanceModelRow> p_instanceModels ) {
		getTableData().clearRowObjects();
		getTableData().getRowObjects().addAll( p_instanceModels );
		tableViewer.setInput( getTableData().getRowObjects() );
		
		updateRemoveButtonEnablement();
	}
	
	public Map<String, String> getDataString() {
		final Map<String, String> dataStr = new HashMap<String, String>();
		
		for ( final InstanceModelRow row : getTableData().getRowObjects() ) {
			final URI outputModelUri = row.getOutputInstanceModel();
			dataStr.put( row.getInputInstanceModel().toString(), outputModelUri == null || outputModelUri.isEmpty() ? null : outputModelUri.toString() );
		}
		
		return dataStr;
	}

	@Override
	protected TableData<Object, InstanceModelRow> createTableData() {
		return new TableData<Object, InstanceModelRow>();
	}
	
	@Override
	protected boolean buttonAddClicked() {
		final String uris = promtInstanceModelFiles();
		
		if ( uris != null ) {
			final StringTokenizer tokenizer = new StringTokenizer( uris, " " );
			
			while ( tokenizer.hasMoreTokens() ) {
				final InstanceModelRow row = new InstanceModelRow( URI.createURI( tokenizer.nextToken() ), null );
				getTableData().addRowObject( row );
			}
			
			tableViewer.refresh();
			tableViewer.getControl().getParent().getParent().layout();
			
			return true;
		}
		
		return false;
	}

	private String promtInstanceModelFiles() {
		final ResourceDialog dialog = new EMFResourceSelectionDialog( 	Display.getCurrent().getActiveShell(),
																		"Add Instance Model(s)" ,
																		SWT.OPEN | SWT.MULTI,
																		null );
		
		if ( dialog.open() == ResourceDialog.OK ) {
			return dialog.getURIText();
		}
		
		return null;
	}
}