package de.mdelab.sdm.interpreter.core.debug;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.logging.Logger;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.net4j.util.om.monitor.SubMonitor;

import de.mdelab.sdm.interpreter.core.SDMException;
import de.mdelab.sdm.interpreter.core.SDMInterpreter;
import de.mdelab.sdm.interpreter.core.executionTrace.ExecutionTrace;
import de.mdelab.sdm.interpreter.core.executionTrace.ExecutionTraceNotificationReceiver;
import de.mdelab.sdm.interpreter.core.notifications.OutputStreamNotificationReceiver;
import de.mdelab.sdm.interpreter.core.variables.Variable;

public abstract class SDEclipseInterpreterLauncher<CoverageReport, Activity extends EObject, Classifier> {
	
	protected final ClassLoader classLoader;

	private SDDebugger<?, Activity, ?, ?, ?> debugger;

	public SDEclipseInterpreterLauncher() {
		this( null );
	}
	
	public SDEclipseInterpreterLauncher( final ClassLoader p_classLoader ) {
		classLoader = p_classLoader == null ? getClass().getClassLoader() : p_classLoader;
		debugger = null;
	}
	
	protected abstract SDMInterpreter<Activity, ?, ?, ?, ?, ?, Classifier, ?, ?> createInterpreter( List<Variable<Classifier>> p_variableBindings )
	throws SDMException;
	
	protected abstract SDDebugger<?, Activity, ?, ?, ?> createDebugger( 	SDMInterpreter<Activity, ?, ?, ?, ?, ?, Classifier, ?, ?> sdi,
																			Integer p_debuggerPort,
																			Integer p_resourceSyncPort );

	protected abstract ExecutionTraceNotificationReceiver<Activity, ?, ?, ?, ?, ?, Classifier, ?, ?> createExecutionTraceNotificationReceiver();
	
	protected abstract OutputStreamNotificationReceiver<Activity, ?, ?, ?, ?, ?, Classifier, ?, ?> createOutputStreamNotificationReceiver( OutputStream p_outputStream );
	
	protected abstract SDCoverageNotificationReceiver<CoverageReport, Classifier> createCoverageNotificationReceiver();

	protected abstract String getName( Activity p_activity );

	public SDExecution<CoverageReport, Classifier> launch( 	final Activity p_activity,
															final List<Variable<Classifier>> p_variableBindings,
															final Integer p_debuggerPort,
															final Integer p_resourceSetSynchronizerPort,
															final boolean pb_debugOutput,
															final boolean pb_execTrace,
															final boolean pb_coverageTrace,
															final IProgressMonitor p_monitor,
															final Logger p_logger,
															final OutputStream p_outputStream )
	throws SDMException {
		final SubMonitor subMonitor = SubMonitor.convert( p_monitor, 3 );

		if ( p_activity == null ) {
			throw new IllegalArgumentException(	"Activity cannot be null" );
		}

		if ( p_logger == null ) {
			throw new IllegalArgumentException(	"Logger cannot be null" );
		}

		/*
		 * Execute
		 */

		try {
			final SDMInterpreter<Activity, ?, ?, ?, ?, ?, Classifier, ?, ?> sdi = createInterpreter( p_variableBindings );

			if ( pb_debugOutput ) {
				/*
				 * Attach an output stream notification receiver
				 */
				sdi.getNotificationEmitter().addNotificationReceiver( createOutputStreamNotificationReceiver( p_outputStream ) );
			}
			else {
				p_logger.info( "Executing activity '" + getName( p_activity ) + "'..." );
			}

			final ExecutionTraceNotificationReceiver<Activity, ?, ?, ?, ?, ?, Classifier, ?, ?> executionTraceNotificationReceiver;

			if ( pb_execTrace ) {
				/*
				 * Attach an execution trace notification receiver
				 */
				executionTraceNotificationReceiver = createExecutionTraceNotificationReceiver();
				sdi.getNotificationEmitter().addNotificationReceiver( executionTraceNotificationReceiver );
			}
			else {
				executionTraceNotificationReceiver = null;
			}

			final SDCoverageNotificationReceiver<CoverageReport, Classifier> coverageNotificationReceiver;

			if ( pb_coverageTrace ) {
				coverageNotificationReceiver = createCoverageNotificationReceiver();
				sdi.getNotificationEmitter().addNotificationReceiver( coverageNotificationReceiver );
			}
			else {
				coverageNotificationReceiver = null;
			}

			if ( p_debuggerPort != null ) {
				/*
				 * Attach an output stream notification receiver
				 */
				//sdi.getNotificationEmitter().addNotificationReceiver( new MLSDMOutputStreamNotificationReceiver( p_outputStream ) );
				debugger = createDebugger( sdi, p_debuggerPort, p_resourceSetSynchronizerPort );
				p_logger.info( "Waiting for connection of debug client (debugger port: " + p_debuggerPort + ")..." );
			}

			final long start = System.nanoTime();
			final Map<String, Variable<Classifier>> returnVariables = sdi.executeActivity( p_activity, p_variableBindings );
			final long stop = System.nanoTime();

			assert returnVariables != null;

			subMonitor.worked( 2 );

			final ExecutionTrace execTrace;
			
			if ( pb_execTrace ) {
				assert executionTraceNotificationReceiver != null;
				
				execTrace = executionTraceNotificationReceiver.getExecutionTrace();
			}
			else {
				execTrace = null;
			}

			final CoverageReport covTrace;
			
			if ( pb_coverageTrace ) {
				assert coverageNotificationReceiver != null;

				covTrace = coverageNotificationReceiver.getCoverageReport();
			}
			else {
				covTrace = null;
			}
				
			logOutputParameterValues( p_activity, returnVariables, p_logger );
//			for ( final ActivityParameter param : p_activity.getParameters() ) {
//				final ActivityParameterDirectionEnum direction = param.getDirection();
//				
//				if ( direction == ActivityParameterDirectionEnum.OUT	|| direction == ActivityParameterDirectionEnum.INOUT ) {
//					p_logger.info( "Output parameter '"	+ param.getName() + "' : " + returnVariables.get( param.getName() ).getValue() );
//				}
//			}
	
			p_logger.info( "Execution time: " + TimeUnit.NANOSECONDS.toMillis( stop - start )	+ " msec" );

			subMonitor.worked( 1 );

			final SDExecution<CoverageReport, Classifier> execResult = createExecutionResult( execTrace, covTrace, returnVariables );
			
			return execResult;
		}
		finally  {
			if ( debugger != null ) {
				try {
					debugger.terminate( true );
					debugger = null;
				}
				catch ( final Exception p_ex ) {
					p_ex.printStackTrace( new PrintStream(  p_outputStream ) );
				}
			}
		}
	}
	
	protected abstract SDExecution<CoverageReport, Classifier> createExecutionResult( 	ExecutionTrace p_execTrace,
																						CoverageReport p_covReport,
																						Map<String, Variable<Classifier>> p_returnVariables );
	
	protected abstract void logOutputParameterValues( 	Activity p_activity,
														Map<String, Variable<Classifier>> p_returnVariables,
														Logger p_logger );
	
	public boolean isReady() {
		return debugger != null && debugger.isInitialized();
	}

	public SDDebugger<?, Activity, ?, ?, ?> getDebugger() {
		return debugger;
	}
}
